'''export a CSV file '''

import csv
import gzip
import os
import shutil


from libgutenberg import GutenbergDatabase
from libgutenberg.DublinCoreMapping import DublinCoreObject
from libgutenberg.Models import Book


FEEDS = os.getenv ('FEEDS') or ''
CSV_FN = 'pg_catalog.csv'
OB = GutenbergDatabase.Objectbase(False)

def books():
    yield ['Text#', 'Type', 'Issued', 'Title', 'Language', 'Authors',
           'Subjects', 'LoCC', 'Bookshelves',]
    session = OB.get_session()
    booknums = session.query(Book.pk).where(Book.pk < 99999).order_by(Book.pk).all()
    for booknum in booknums:
        dc = DublinCoreObject()
        dc.load_from_database(booknum[0])
        if not dc.book:
            continue
        row = [dc.book.pk,
               '; '.join([dcmitype.id for dcmitype in dc.dcmitypes]),
               dc.book.release_date.isoformat(),
               dc.title,
               '; '.join([lang.id for lang in dc.languages]),
               '; '.join([dc.format_author_date_role(author) for author in dc.authors]),
               '; '.join([subject.subject for subject in dc.subjects]),
               '; '.join([locc.id for locc in dc.loccs]),
               '; '.join([shelf.bookshelf for shelf in dc.bookshelves]),
              ]
        yield row


def main():
    fn = os.path.join(FEEDS, CSV_FN)
    with open(fn, 'w', newline='') as csvfile:
        csvwriter = csv.writer(csvfile)
        for row in books():
            csvwriter.writerow(row)
    with open(fn, 'rb') as f_in:
        with gzip.open(fn + '.gz', 'wb') as f_out:
            shutil.copyfileobj(f_in, f_out)



#!/usr/bin/env python3
"""
Allows etext-specific messages to be sent to an address registered for that etext.

uses a json file to save notification addresses.


"""

from datetime import datetime
import json
import os
import re
import stat
import smtplib

from email.message import EmailMessage

from libgutenberg.Logger import debug, error, info, warning # pylint: disable=unused-import


from ebookmaker.CommonCode import NOTIFICATION_DIR

PRIVATE = os.getenv('PRIVATE') or ''
NOTIFY_FILE = os.path.join(PRIVATE, 'logs', 'json', 'notify.json')
ARCHIVE_DIR = os.path.join(PRIVATE,'logs', 'json', 'backup')
SMTP_HOST = os.getenv('SMTP_HOST') or 'localhost'
SMTP_USER = os.getenv('SMTP_USER') or ''
SMTP_PASSWORD = os.getenv('SMTP_PASSWORD') or ''
REPLY_TO_EMAIL = os.getenv('REPLY_TO_EMAIL') or 'help2021@pglaf.org'
SMTP_SSL = os.getenv('SMTP_SSL') or False

NOTIFY_PROCESSED = """
Ebook #{ebook} has been processed and ebook files have been posted successfully at Project Gutenberg.
"""
NOTIFY_ISSUE = """
Ebook #{ebook} has been processed and ebook files have been posted at Project Gutenberg. The posting team has been notified of a possible issue with the processing. You will be notified again when the issue is resolved.
"""
NOTIFY_CRITICAL = """
Ebook #{ebook} has been processed and posted at Project Gutenberg. There may exist a critical issue with the ebook processing. Here is the log entry:
{records}

The full processing log for this ebook may be downloaded at https://www.gutenberg.org/cache/epub/{ebook}/pg{ebook}.converter.log
"""


class AddressBook:
    """
        address_book is a dict of dicts, one dict per book.
        each book has email addresses by role, default role is 'notify', usually the submitter.
        other roles are 'ww' for the white washer managing the submission.
        and other roles that may be invented in the future
    """
    def __init__(self):
        self._address_book = None
        
    @property
    def address_book(self):
        if self._address_book == None:
            try:
                with open(NOTIFY_FILE, 'r') as json_file:
                    self._address_book = json.loads(json_file.read())
            except FileNotFoundError:
                error('could not find %s', NOTIFY_FILE)
                self._address_book = {}
                self.save()
        return self._address_book

    def set_email(self, ebook, email, role='notify'):
        if not email:
            return False
        try:
            ebook = str(int(ebook))
        except ValueError:
            return False
        if ebook in self.address_book:
            addresses = self.address_book[ebook]
        else:
            addresses = {}
            self.address_book[ebook] = addresses
        if email and not "@" in email:
            email = email.strip() + "@pglaf.org"
        addresses[role] = email
        self.save()
        return True

    def get_email(self, ebook, role='notify'):
        try:
            ebook = str(int(ebook))
        except ValueError:
            return ''
        return self.address_book.get(ebook, {}).get(role, '')

    def save(self):
        """ save the data before destroying the object """
        with open(NOTIFY_FILE, 'w+') as json_file:
            json.dump(self.address_book, json_file)


ADDRESS_BOOK = AddressBook()

def notify(ebook, message, role='notify', subject='Gutenberg backend notifications'):
    info(message)
    address = ADDRESS_BOOK.get_email(ebook, role=role)
    if not address:
        # archive unsent message
        message_archive = '{}/{}.messages'.format(ARCHIVE_DIR, ebook)
        now = datetime.now().isoformat()
        with open(message_archive, 'a+') as messagefile:
            messagefile.write('%s: %s\nSubject: %s\n%s\n' % (role, now, subject, message))
        return 1
    try:
        if SMTP_SSL:
            server = smtplib.SMTP_SSL(SMTP_HOST)
        else:
            server = smtplib.SMTP(SMTP_HOST)
        if SMTP_PASSWORD:
            server.login(SMTP_USER, SMTP_PASSWORD)
        msg = EmailMessage()
        msg.set_content(message)
        msg['Subject'] = subject
        msg['From'] = REPLY_TO_EMAIL
        msg['To'] = address
        server.send_message(msg)
        server.quit()
        return 2
    except ConnectionError as e:
        error(e)
        return 0


def send_notifications(ebooks=None):
    """ critical notifications are generated by ebookconverter logging
        ebooks should be a list of processed ebooks.
    """
    ebooks = ebooks if ebooks else []
    criticals = []
    for filename in sorted(os.listdir(NOTIFICATION_DIR)):
        mode = os.stat(os.path.join(NOTIFICATION_DIR, filename))[stat.ST_MODE]
        if stat.S_ISDIR(mode):
            continue
        m = re.match(r'^(\d+)\.txt$', filename)
        if m:
            ebook = int(m.group(1))
            criticals.append(ebook)
            queued_file = os.path.join(NOTIFICATION_DIR, filename)
            records = ''
            with open(queued_file, 'r') as messagefile:
                records = messagefile.read()
            if records:
                message = NOTIFY_CRITICAL.format(ebook=ebook, records=records)
                code = notify(ebook, message, role='ww')
                if code:
                    os.remove(queued_file)

    # send 'processed' notifications to notification address
    for ebook in ebooks:
        try:
            ebook = int(ebook)
        except ValueError:
            continue
        if ebook in criticals:
            message = NOTIFY_ISSUE.format(ebook=ebook)
        else:
            message = NOTIFY_PROCESSED.format(ebook=ebook)
        notify(ebook, message)



#!/usr/bin/env python3
#  -*- mode: python; indent-tabs-mode: nil; -*- coding: UTF8 -*-

"""

AutoDelete.py

Copyright 2009 by Marcello Perathoner

Distributable under the GNU General Public License Version 3 or newer.

Looks for any ebooks with changed files in the last X days and then
checks if any of the known files of that ebook have been deleted.


"""

import os

from six.moves import builtins
from six.moves import configparser

from libgutenberg.Logger import debug, info
from libgutenberg import Logger
from libgutenberg import GutenbergGlobals as gg
from libgutenberg import GutenbergDatabase

from ebookmaker.CommonCode import Options

from ebookconverter.EbookConverter import config

PRIVATE = os.getenv ('PRIVATE') or ''
PUBLIC  = os.getenv ('PUBLIC')  or ''

DIRS  = PUBLIC + '/dirs'

options = Options()

def check_book (ebook):
    """ Check all files of ebook for presence. """

    c  = GutenbergDatabase.DB.get_cursor ()
    c2 = GutenbergDatabase.DB.get_cursor ()

    c.execute ("select filename from files where fk_books = %(ebook)s and diskstatus != 5",
               { 'ebook': ebook })

    for row in c.fetchall ():
        row = GutenbergDatabase.xl (c, row)
        try:
            if row.filename.startswith ('/'):
                os.stat (row.filename)
                continue
            if row.filename.startswith ('cache'):
                os.stat (os.path.join (PUBLIC, row.filename))
                continue
            if row.filename.startswith ('dirs'):
                os.stat (os.path.join (PUBLIC, row.filename))
                continue
            os.stat (os.path.join (DIRS, row.filename))
        except OSError:
            c2.execute ("start transaction")
            c2.execute ("update files set diskstatus = 5 where filename = %(filename)s",
                        { 'filename': row.filename })
            c2.execute ("commit")
            info ("Removed from database: %s" % row.filename)


def main ():
    goback = 1
    try:
        config ()
    except configparser.Error as what:
        error ("Error in configuration file: %s", str (what))
        return 1

    Logger.setup (Logger.LOGFORMAT, 'autodelete.log')
    Logger.set_log_level (2)

    debug ("Starting AutoDelete.py")

    GutenbergDatabase.DB = GutenbergDatabase.Database ()
    GutenbergDatabase.DB.connect ()
    c  = GutenbergDatabase.DB.get_cursor ()

    c.execute ("select distinct fk_books from files "
               "where filemtime >= now () - interval '%(goback)s days' "
               "and filename !~ '^cache/' "
               "and diskstatus != 5 and fk_books is not null",
               { 'goback': goback } )

    for row in c.fetchall ():
        row = GutenbergDatabase.xl (c, row)
        ebook = int (row.fk_books)
        Logger.ebook = ebook
        debug ("Checking ebook")
        check_book (ebook)

    Logger.ebook = 0
    debug ("Done AutoDelete.py")


if __name__ == '__main__':
    main ()



#!/usr/bin/env python
#  -*- mode: python; indent-tabs-mode: nil; -*- coding: utf-8 -*-

"""
TwitterWriter.py

Copyright 2010,2014 by Marcello Perathoner

Distributable under the GNU General Public License Version 3 or newer.

Cackle about new ebooks.

"""

from __future__ import unicode_literals

import os

import requests_oauthlib

from requests import RequestException
from oauthlib.oauth1.rfc5849.errors import OAuth1Error

from libgutenberg.Logger import exception
from libgutenberg.GutenbergGlobals import SkipOutputFormat
from libgutenberg import GutenbergDatabase

from ebookmaker import writers


URL = "https://api.twitter.com/1.1/statuses/update.json"

class Writer (writers.BaseWriter):
    """ Class to cackle about new ebooks.

    """

    __instance = None

    def __new__ (cls):
        """ Make Writer a singleton. """

        if cls.__instance is None:
            cls.__instance = object.__new__ (cls)
        return cls.__instance


    def __init__ (self):
        if hasattr (self, 'session'):
            # already initialized
            return

        super (Writer, self).__init__ ()

        get = os.environ.get

        if not get ('TWITTER_API_SECRET'):
            raise SkipOutputFormat ('Twitter credentials not found.')

        self.session = requests_oauthlib.OAuth1Session (
            get ('TWITTER_API_KEY'),
            client_secret = get ('TWITTER_API_SECRET'),
            resource_owner_key = get ('TWITTER_ACCESS_TOKEN'),
            resource_owner_secret = get ('TWITTER_ACCESS_TOKEN_SECRET')
        )


    def build (self, job):
        """ Cackle. """

        if job.dc.project_gutenberg_id < 45400:
            # don't cackle about old books
            return

        db = GutenbergDatabase.Database ()
        db.connect ()
        c = db.get_cursor ()

        c.execute ("select * from tweets where media = 'Twitter' and fk_books = %(fk_books)s",
                   { 'fk_books': job.dc.project_gutenberg_id })

        rows = c.fetchone ()
        if rows:
            # already cackled
            return

        try:
            title = job.dc.make_pretty_title (74)
            tweet = "New #ebook @gutenberg_org: %s https://www.gutenberg.org/ebooks/%d" % (
                title, job.dc.project_gutenberg_id)

            r = self.session.post (URL, data = { 'status': tweet })
            r.raise_for_status ()

            c.execute ('start transaction')
            c.execute (
                """\
                insert into tweets (fk_books, media, time)
                values (%(fk_books)s, 'Twitter', now ())
                """,
                       { 'fk_books': job.dc.project_gutenberg_id })
            c.execute ('commit')

        except (RequestException, OAuth1Error) as what:
            exception ('TwitterWriter: could not cackle about ebook: %s' % what)

        except (GutenbergDatabase.DatabaseError) as what:
            c.execute ('rollback')
            exception ('TwitterWriter: could not write to database (#%s)' % what)



#
# ebookconverter distribution
#

from setuptools import setup

VERSION = '0.7.9'

setup (
    name = 'ebookconverter',
    version = VERSION,

    packages = [
        'ebookconverter',
        'ebookconverter.writers',
    ],

    scripts = [
        'scripts/autodelete',
        'scripts/ebookconverter',
        'scripts/fileinfo',
        'scripts/make_csv',
        'scripts/cron-rebuild-files.sh',
        'scripts/cron-dopush-social.sh',
        'scripts/cron-dopush.sh',
    ],

    install_requires = [
        'ebookmaker>=0.11.16',
        'setproctitle==1.1.10',
        'requests_oauthlib>=1.2.0',
        'rdflib>=4.2.2',
        'qrcode>=6.1',
        'libgutenberg[postgres]>=0.8.11',
    ],
    
    package_data = {
    },

    data_files = [
        ('', ['CHANGES', 'README.md']),
    ],

    # metadata for upload to PyPI

    author = "Eric Hellman",
    author_email = "eric@hellman.org",
    description = "The Project Gutenberg tool to orchestrate ebook generation.",
    long_description = open ('README.md').read(),
    license = "GPL v3",
    url = "https://github.com/gutenbergtools/ebookconverter/",

    classifiers = [
        "License :: OSI Approved :: GNU General Public License (GPL)",
        "Environment :: Console",
        "Operating System :: OS Independent",
        "Intended Audience :: Other Audience",
        "Development Status :: 4 - Beta",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.6",
    ],

    platforms = 'OS-independent'
)



#!/usr/bin/env python
#  -*- mode: python; indent-tabs-mode: nil; -*- coding: UTF8 -*-

"""

QRCodeWriter.py

Copyright 2013-2020 by Project Gutenberg

Distributable under the GNU General Public License Version 3 or newer.

Writes QRCodes.

"""

import os

import qrcode

from libgutenberg.Logger import info, exception
from ebookmaker import writers


DESKTOP_URL = 'https://www.gutenberg.org/ebooks/%d'

class MyQRCode(qrcode.QRCode):
    """ QRCode with parameters. """

    def __init__(self):
        super(MyQRCode, self).__init__(
            error_correction=qrcode.constants.ERROR_CORRECT_M,
            box_size=4,
            border=0,
        )


class Writer(writers.BaseWriter):
    """ QRCode image writer. """

    def build(self, job):
        """ write qrcode . """

        info("Making   %s" % job.outputfile)

        try:
            qr = MyQRCode()
            qr.add_data(DESKTOP_URL % job.ebook)
            qr.make(fit=True)
            qrcode1 = qr.make_image(fill_color="black", back_color="white")

            fn = os.path.join(job.outputdir, job.outputfile)
            with open(fn, 'wb') as fp:
                qrcode1.save(fp, 'png')

            info("Done     %s" % job.outputfile)

        except Exception as what:
            exception("Error building QR-Code: %s" % what)
            raise
